# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------
from pixiedust.display.display import *
import pixiedust_flightpredict
from pixiedust.utils.shellAccess import ShellAccess
from pixiedust_flightpredict.training.training import *
from pixiedust.utils.template import PixiedustTemplateEnvironment

ICON_FAILED = "fa-times"
ICON_SUCCESS = "fa-check"
ICON_INFO = "fa-info"

DFTrainingVarName = 'trainingData'
LabeledRDDTrainingVarName = 'labeledTrainingData'
LabeledRDDTestVarName = 'labeledTestData'


class ConfigureTraining(Display):
    def loadTemplate(self, templateName, **kwargs):
         return '\\"' + self.renderTemplate(templateName,**kwargs).replace('\n','\\\\n').replace('"', '\\\\\\"') + '\\"'

    def doRender(self, handlerId):
        self.addProfilingTime = False
        #update the configuration
        pixiedust_flightpredict.Configuration.update( **self.options )

        steps=[
            {"title": "Architecture", "template": "step_welcome.html"},
            {"title": "Credentials", "template": "step_credentials.html","args":[
                ("cloudantHost","Cloudant Host"),("cloudantUserName","Cloudant User Name"),("cloudantPassword","Cloudant Password"),
                ("weatherUrl", "Weather URL"),("appId", "FlightStats API Id"),("appKey", "FlightStats API Key")
            ]},
            {"title": "Training Set", "template": "step_sets.html", "args":[
                ('Database Name', 'TrainingDbName',''),
                ('SQL Table Name', 'TrainingSQLTableName','training'),
                ('DataFrame Variable Name', 'DFTrainingVarName', DFTrainingVarName), 
                ('LabeledRDD Variable Name', 'LabeledRDDTrainingVarName', LabeledRDDTrainingVarName)
            ]},
            {"title": "Test Set", "template": "step_sets.html", "args":[
                ('Database Name', 'TestDbName',''),
                ('SQL Table Name', 'TestSQLTableName','test'), 
                ('DataFrame Variable Name', 'DFTestVarName','testData'), 
                ('LabeledRDD Variable Name', 'LabeledRDDTestVarName', LabeledRDDTestVarName)
            ]}
        ]

        if self.options.get("nostore_edit")=='true':
            self._addHTMLTemplate("configureWizard.html", steps=steps);
        else:
            tasks=[
                self.checkConfigParams(["cloudantHost","cloudantUserName","cloudantPassword"], "Cloudant Configuration is OK"),
                self.checkConfigParams(["weatherUrl"], "WeatherUrl Configuration is OK"),
                self.checkConfigParams(["appId", "appKey"], "FlightStats Configuration is OK"),
                self.checkDataSet( steps[2]["args"] ),
                self.checkDataSet( steps[3]["args"] ),
                self.checkLabeledRDD( LabeledRDDTrainingVarName, pixiedust_flightpredict.Configuration.TrainingSQLTableName ),
                self.checkLabeledRDD( LabeledRDDTestVarName, pixiedust_flightpredict.Configuration.TestSQLTableName  ),
                self.checkCustomHandlers(),
                self.checkModels()
            ]
            self._addHTMLTemplate("configureWizard.html", tasks = tasks )

    def checkConfigParams(self, fields, okDescription=''):
        for f in fields:
            if pixiedust_flightpredict.Configuration[f] is None:
                return { "status-class": ICON_FAILED,
                         "task": "Missing configuration {0}".format(f),
                         "action": "Click on the edit configuration button and provide the require parameter"
                        }
        return { "status-class": ICON_SUCCESS,
                 "task": okDescription,
                 "action": "None"
                }

    def checkCustomHandlers(self):
        customHandlers = [x for x in ShellAccess if isinstance(ShellAccess[x], defaultTrainingHandler)]
        return { "status-class": ICON_INFO,
                 "task": "{0} custom Handler found: {1}".format(len(customHandlers), reduce(lambda x,y: x + " " + y, customHandlers, "")),
                 "action": "Create a new custom Handler",
                 "code": "get_ipython().set_next_input({0})".format(self.loadTemplate("gencode/customHandlers.template")),
                 "id": "customHandlers"
                }

    def checkModels(self):
        models = pixiedust_flightpredict.Configuration.getModels()
        models = models if len(models)==0 else zip(*models)[0]
        return {
            "status-class": ICON_INFO,
            "task": "{0} model(s) have been created. {1}".format(len(models), reduce(lambda x,y: x + ", " + y, models, "")),
            "action": "Run the cells that build your models"
        }

    def checkLabeledRDD(self, varName, sqlTableName):
        if ShellAccess[varName] is None:
            return { "status-class": ICON_FAILED,   
                    "task": "The LabeledRDD {0} is not defined. If you already have a cell that loads it, please run it now. If not, click on the action button to generate a new cell".format(varName),
                    "action": "Generate Cell code to load {0}".format(varName),
                    "code": "get_ipython().set_next_input({0})".format(self.loadTemplate("gencode/labeledRDD.template", varName=varName, sqlTableName = sqlTableName)),
                    "id": varName
                    }

        return { "status-class": ICON_SUCCESS,
                 "task": "LabeledRDD {0} correctly loaded".format(varName),
                 "action": "None"
                }

    def checkDataSet(self, fieldInfos):
        task = self.checkConfigParams([f[1] for f in fieldInfos])
        if task["status-class"] == ICON_FAILED:
            return task
        
        DbName = pixiedust_flightpredict.Configuration[fieldInfos[0][1]]
        SQLTableVarName = pixiedust_flightpredict.Configuration[fieldInfos[1][1]]
        DFVarName = pixiedust_flightpredict.Configuration[fieldInfos[2][1]]

        if ShellAccess[DFVarName] is None:
            code = """ \\"dbName='{dbName}'\\\\n{varName} = pixiedust_flightpredict.loadDataSet(dbName,'{sqlTableName}')\\\\ndisplay({varName})\\" """\
                .format(varName=DFVarName,dbName=DbName, sqlTableName=SQLTableVarName)
            return { "status-class": ICON_FAILED,   
                    "task": "The variable {0} is not defined. If you already have a cell that loads it, please run it now. If not, click on the action button to generate a new cell".format(DFVarName),
                    "action": "Generate Cell code to load {0}".format(DFVarName),
                    "code": "get_ipython().set_next_input({0})".format(code),
                    "id": DFVarName
                    }

        return { "status-class": ICON_SUCCESS,
                 "task": "dataframe {0} correctly loaded".format(DFVarName),
                 "action": "None"
                }



# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------
import requests
from requests.auth import HTTPBasicAuth
from pixiedust_flightpredict import Configuration
from pixiedust.utils.shellAccess import ShellAccess
import pixiedust
import dateutil.parser as parser
from pixiedust.utils.template import *

myLogger = pixiedust.getLogger(__name__)

historyDBName = "flightpredictorhistory"

env = PixiedustTemplateEnvironment()

def saveFlightResults(payload):
    auth=HTTPBasicAuth(Configuration.cloudantUserName, Configuration.cloudantPassword)

    #make sure the database is created first
    url = Configuration.cloudantHost
    if "://" not in url:
        url = "https://"+url
    r = requests.get(url + "/" + historyDBName, auth=auth)
    if r.status_code != 200:
        if r.status_code == 404:
            r = requests.put(url+ "/" + historyDBName, auth=auth)
            if r.status_code != 200 and r.status_code != 201 and r.status_code != 412:
                return myLogger.error("Error creating to the history database: {0}".format(r.text))
        else:
            return myLogger.error("Error connecting to the history database: {0}".format(r.text))

    depAirportInfo = payload["departureAirportInfo"]["airportInfo"]
    arrAirportInfo = payload["arrivalAirportInfo"]["airportInfo"]
    r = requests.post(url+"/" + historyDBName, auth=auth,json={
        'depAirportFSCode': depAirportInfo["fs"],
        'depAirportName': depAirportInfo["name"],
        'depAirportLong': depAirportInfo["longitude"],
        'depAirportLat': depAirportInfo["latitude"],
        'arrAirportFSCode': arrAirportInfo["fs"],
        'arrAirportName': arrAirportInfo["name"],
        'arrAirportLong': arrAirportInfo["longitude"],
        'arrAirportLat': arrAirportInfo["latitude"],
        'prediction': payload["prediction"],
        'carrierFsCode': payload["flightInfo"]["carrierFsCode"],
        'flightNumber': payload["flightInfo"]["flightNumber"],
        'departureDate': parser.parse( payload["flightInfo"]["departureTime"]).strftime("%B %d %Y"),
        'departureTime': parser.parse( payload["flightInfo"]["departureTime"]).strftime("%I:%M %p")
    })
    if r.status_code != 200 and r.status_code != 201:
        return myLogger.error("Error saving flight results in database to the history database: {0}".format(r.text))
"""
Load the flight history into a dataFrame
"""
def loadFlightHistory():
    if Configuration.cloudantHost is None or Configuration.cloudantUserName is None or Configuration.cloudantPassword is None:
        raise Exception("Missing credentials")
    return ShellAccess.sqlContext.read.format("com.cloudant.spark")\
        .option("cloudant.host",Configuration.cloudantHost)\
        .option("cloudant.username",Configuration.cloudantUserName)\
        .option("cloudant.password",Configuration.cloudantPassword)\
        .option("schemaSampleSize", "-1")\
        .load(historyDBName)

def getBadgeHtml(depAirportCode, arrAirportCode):
    if ShellAccess.flightHistoryDF is None:
        myLogger.info("Reloading flight history from getBadgeHtml")
        ShellAccess.flightHistoryDF = loadFlightHistory()
    
    df = ShellAccess.flightHistoryDF
    res = df.filter( df['depAirportFSCode']== depAirportCode )\
        .filter( df['arrAirportFSCode']==arrAirportCode )\
        .map(lambda t: ((t["carrierFsCode"], t["flightNumber"], t["departureDate"],t["departureTime"], t["depAirportName"], t["arrAirportName"]), 1))\
        .reduceByKey(lambda t,v : t+v)
    
    return env.getTemplate("flightBadge.html").render(flights=res.collect())





# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------

from .flightAccess import *
from .weatherAccess import *
from .flightHistory import *
from pixiedust_flightpredict import Configuration
from pixiedust_flightpredict.training import *
from collections import Counter
import pixiedust
import json

myLogger = pixiedust.getLogger(__name__)

"""
{'appendix': {'airlines': [{'active': True,
    'fs': 'NK',
    'iata': 'NK',
    'icao': 'NKS',
    'name': 'Spirit Airlines',
    'phoneNumber': '800-772-7117'}],
  'airports': [{'active': True,
    'city': 'Boston',
    'cityCode': 'BOS',
    'classification': 1,
    'countryCode': 'US',
    'countryName': 'United States',
    'elevationFeet': 19,
    'faa': 'BOS',
    'fs': 'BOS',
    'iata': 'BOS',
    'icao': 'KBOS',
    'latitude': 42.36646,
    'localTime': '2016-10-03T23:57:34.128',
    'longitude': -71.020176,
    'name': 'Logan International Airport',
    'postalCode': '02128-2909',
    'regionName': 'North America',
    'stateCode': 'MA',
    'street1': 'One Harborside Drive',
    'street2': '',
    'timeZoneRegionName': 'America/New_York',
    'utcOffsetHours': -4.0,
    'weatherZone': 'MAZ015'},
   {'active': True,
    'city': 'Las Vegas',
    'cityCode': 'LAS',
    'classification': 1,
    'countryCode': 'US',
    'countryName': 'United States',
    'elevationFeet': 2095,
    'faa': 'LAS',
    'fs': 'LAS',
    'iata': 'LAS',
    'icao': 'KLAS',
    'latitude': 36.081,
    'localTime': '2016-10-03T20:57:34.128',
    'longitude': -115.147599,
    'name': 'McCarran International Airport',
    'postalCode': '89119',
    'regionName': 'North America',
    'stateCode': 'NV',
    'street1': '5757 Wayne Newton Boulevard',
    'timeZoneRegionName': 'America/Los_Angeles',
    'utcOffsetHours': -7.0,
    'weatherZone': 'NVZ020'}],
  'equipments': [{'iata': '32S',
    'jet': True,
    'name': 'Airbus A318 / A319 / A320 / A321',
    'regional': False,
    'turboProp': False,
    'widebody': False}]},
 'request': {'carrier': {'fsCode': 'NK', 'requestedCode': 'NK'},
  'codeType': {},
  'date': {'day': '12',
   'interpreted': '2016-10-12',
   'month': '10',
   'year': '2016'},
  'departing': True,
  'flightNumber': {'interpreted': '641', 'requested': '641'},
  'url': 'https://api.flightstats.com/flex/schedules/rest/v1/json/flight/NK/641/departing/2016/10/12'},
 'scheduledFlights': [{'arrivalAirportFsCode': 'LAS',
   'arrivalTerminal': '1',
   'arrivalTime': '2016-10-12T19:34:00.000',
   'carrierFsCode': 'NK',
   'codeshares': [],
   'departureAirportFsCode': 'BOS',
   'departureTerminal': 'B',
   'departureTime': '2016-10-12T16:40:00.000',
   'flightEquipmentIataCode': '32S',
   'flightNumber': '641',
   'isCodeshare': False,
   'isWetlease': False,
   'referenceCode': '1874-588916--',
   'serviceClasses': ['R', 'Y'],
   'serviceType': 'J',
   'stops': 0,
   'trafficRestrictions': []}]}
"""

def runModel(flight, date, departureAirport=None):
    if not Configuration.isReadyForRun():
        raise ValueError("Unable to predict flight delay because no models are available. Please run configure for more details")

    if departureAirport is None:
        departureAirport = "LAS"

    response = getFlightSchedule(flight, date, departureAirport)
    myLogger.debug("Schedule for flight {0} at date {1} : {2}".format(flight, date, response ))
    if "error" in response:
        return {"error": "Unable to access schedule {0}".format(response["error"])}

    payload={}
    appendix=response['appendix']
    payload["flightInfo"]=scheduledFlight=response["scheduledFlight"]

    def getAirportJSON(code):
        for airport in appendix["airports"]:
            if airport['fs'] == code:
                return airport
    
    payload["departureAirportInfo"]=departureInfo={}
    departureInfo["airportInfo"]=depAirportJSON=getAirportJSON( scheduledFlight['departureAirportFsCode'] )
    departureInfo["weatherForecast"]= depWeather = getWeather(depAirportJSON['latitude'], depAirportJSON['longitude'], scheduledFlight["departureTimeUTC"])

    payload["arrivalAirportInfo"]=arrivalInfo={}
    arrivalInfo["airportInfo"]=arrAirportJSON=getAirportJSON( scheduledFlight['arrivalAirportFsCode'] )
    arrivalInfo["weatherForecast"]= arrWeather=getWeather(arrAirportJSON['latitude'], arrAirportJSON['longitude'], scheduledFlight["arrivalTimeUTC"] )

    payload["prediction"]=prediction = {}
    prediction["models"]=predictionModels = []
    #create the features vector
    features = createFeaturesVector(depWeather, arrWeather, scheduledFlight, depAirportJSON, arrAirportJSON)
    #count the predicted classes to make an overall prediction
    predictedClasses = []
    trainingHandler = getTrainingHandler();
    for modelVar,model in Configuration.getModels():
        p = model.predict(features)
        predictedClasses.append(p)
        predictionModels.append({
            "model": model.__class__.__name__,
            "prediction": trainingHandler.getClassLabel(p)
        })
    classes = Counter(predictedClasses).most_common()
    if len(classes)>1 and classes[0][1]==classes[1][1]:
        prediction["overall"] = "Undecided: {0}/{1}".format(trainingHandler.getClassLabel(classes[0][0]), trainingHandler.getClassLabel(classes[1][0]))
    else:
        prediction["overall"] = trainingHandler.getClassLabel(classes[0][0])
    """
    payload["prediction"]={
        "overall": "Delayed between 13 and 41 minutes",
        "models":[
            {"model":"NaiveBayesModel", "prediction":"Delayed between 13 and 41 minutes"},
            {"model":"DecisionTreeModel: Delayed between 13 and 41 minutes"},
            {"model":"LogisticRegressionModel: Delayed between 13 and 41 minutes"},
            {"model":"RandomForestModel: Delayed between 13 and 41 minutes"}
        ]
    }
    """

    myLogger.debug("runModel Payload {0}".format(payload))
    try:
        saveFlightResults(payload)
    except:
        myLogger.exception("Unable to save flight results {0}".format(payload))

    return json.dumps(payload)
    #payload format
    """
    {
        "flightInfo": <flightInfo>,
        "departureAirportInfo":{
            "airportInfo": <airport>
            "weatherForecast": <weather>
        },
        "arrivalAirportInfo":{
            "airportInfo": <airport>
            "weatherForecast": <weather>
        },
        "prediction": {
            "overall": <prediction>,
            "models":[
                { 
                    "model": <modelName>
                    "prediction": <prediction>
                },
                ...
            ]
        }
    }
    """

def mapAttribute(attr):
    if attr=="dewPt":
        return "dewpt"
    return attr

def createFeaturesVector(depWeather, arrWeather, scheduledFlight, arrAirport, depAirport):
    features=[]
    for attr in attributes:
        features.append(depWeather[mapAttribute(attr)])
    for attr in attributes:
        features.append(arrWeather[mapAttribute(attr)])
    
    #Call training handler for custom features
    s=type('dummy', (object,), {
        'departureTime':scheduledFlight["departureTime"], 'arrivalTime':scheduledFlight["arrivalTime"], 'arrivalAirportFsCode': arrAirport["fs"], 
        'departureAirportFsCode':depAirport["fs"],'departureWeather': depWeather, 'arrivalWeather': arrWeather})

    myLogger.debug("Creating features vector with {0}".format(s))
    for value in getTrainingHandler().customTrainingFeatures(s):
        features.append( value )
    
    return features

def runModelTest(flight, date, departureAirport=None):
    payload = {
        'arrivalAirportInfo': {
            'airportInfo': {
                'active': True,
                'city': 'Las Vegas',
                'cityCode': 'LAS',
                'classification': 1,
                'countryCode': 'US',
                'countryName': 'United States',
                'elevationFeet': 2095,
                'faa': 'LAS',
                'fs': 'LAS',
                'iata': 'LAS',
                'icao': 'KLAS',
                'latitude': 36.081,
                'localTime': '2016-10-04T12:28:48.884',
                'longitude': -115.147599,
                'name': 'McCarran International Airport',
                'postalCode': '89119',
                'regionName': 'North America',
                'stateCode': 'NV',
                'street1': '5757 Wayne Newton Boulevard',
                'timeZoneRegionName': 'America/Los_Angeles',
                'utcOffsetHours': -7.0,
                'weatherZone': 'NVZ020'
            },
            'weatherForecast': {
                'class': 'fod_short_range_hourly',
                'clds': 0,
                'day_ind': 'D',
                'dewpt': -9,
                'dow': 'Tuesday',
                'expire_time_gmt': 1475609398,
                'fcst_valid': 1475622000,
                'fcst_valid_local': '2016-10-04T16:00:00-0700',
                'feels_like': 26,
                'golf_category': 'Very Good',
                'golf_index': 9,
                'gust': None,
                'hi': 26,
                'icon_code': 32,
                'icon_extd': 3200,
                'mslp': 1006.5,
                'num': 4,
                'phrase_12char': 'Sunny',
                'phrase_22char': 'Sunny',
                'phrase_32char': 'Sunny',
                'pop': 0,
                'precip_type': 'rain',
                'qpf': 0.0,
                'rh': 9,
                'severity': 1,
                'snow_qpf': 0.0,
                'subphrase_pt1': 'Sunny',
                'subphrase_pt2': '',
                'subphrase_pt3': '',
                'temp': 26,
                'uv_desc': 'Low',
                'uv_index': 2,
                'uv_index_raw': 1.7,
                'uv_warning': 0,
                'vis': 16.0,
                'wc': 26,
                'wdir': 112,
                'wdir_cardinal': 'ESE',
                'wspd': 8,
                'wxman': 'wx1000'
            }
        },
        'departureAirportInfo': {
            'airportInfo': {
                'active': True,
                'city': 'Boston',
                'cityCode': 'BOS',
                'classification': 1,
                'countryCode': 'US',
                'countryName': 'United States',
                'elevationFeet': 19,
                'faa': 'BOS',
                'fs': 'BOS',
                'iata': 'BOS',
                'icao': 'KBOS',
                'latitude': 42.36646,
                'localTime': '2016-10-04T15:28:48.884',
                'longitude': -71.020176,
                'name': 'Logan International Airport',
                'postalCode': '02128-2909',
                'regionName': 'North America',
                'stateCode': 'MA',
                'street1': 'One Harborside Drive',
                'street2': '',
                'timeZoneRegionName': 'America/New_York',
                'utcOffsetHours': -4.0,
                'weatherZone': 'MAZ015'
            },
            'weatherForecast': {
                'class': 'fod_short_range_hourly',
                'clds': 29,
                'day_ind': 'D',
                'dewpt': 10,
                'dow': 'Tuesday',
                'expire_time_gmt': 1475609399,
                'fcst_valid': 1475611200,
                'fcst_valid_local': '2016-10-04T16:00:00-0400',
                'feels_like': 14,
                'golf_category': 'Very Good',
                'golf_index': 8,
                'gust': None,
                'hi': 15,
                'icon_code': 34,
                'icon_extd': 3400,
                'mslp': 1027.1,
                'num': 1,
                'phrase_12char': 'M Sunny',
                'phrase_22char': 'Mostly Sunny',
                'phrase_32char': 'Mostly Sunny',
                'pop': 0,
                'precip_type': 'rain',
                'qpf': 0.0,
                'rh': 69,
                'severity': 1,
                'snow_qpf': 0.0,
                'subphrase_pt1': 'Mostly',
                'subphrase_pt2': 'Sunny',
                'subphrase_pt3': '',
                'temp': 15,
                'uv_desc': 'Low',
                'uv_index': 1,
                'uv_index_raw': 1.31,
                'uv_warning': 0,
                'vis': 16.0,
                'wc': 14,
                'wdir': 54,
                'wdir_cardinal': 'NE',
                'wspd': 24,
                'wxman': 'wx1000'
            }
        },
        'flightInfo': {
            'arrivalAirportFsCode': 'LAS',
            'arrivalTerminal': '1',
            'arrivalTime': '2016-10-12T19:34:00.000',
            'carrierFsCode': 'NK',
            'codeshares': [],
            'departureAirportFsCode': 'BOS',
            'departureTerminal': 'B',
            'departureTime': '2016-10-12T16:40:00.000',
            'flightEquipmentIataCode': '32S',
            'flightNumber': '641',
            'isCodeshare': False,
            'isWetlease': False,
            'referenceCode': '1875-576194--',
            'serviceClasses': ['R', 'Y'],
            'serviceType': 'J',
            'stops': 0,
            'trafficRestrictions': []
        },
        'prediction': {
            'models': [
                {'model': 'NaiveBayesModel', 'prediction': 'Delayed between 13 and 41 minutes'},
                {'model': 'DecisionTreeModel', 'prediction': 'Delayed between 13 and 41 minutes'},
                {'model': 'LogisticRegressionModel', 'prediction': 'Delayed between 13 and 41 minutes'},
                {'model': 'RandomForestModel', 'prediction': 'Delayed between 13 and 41 minutes'}
            ],
            'overall': 'Delayed between 13 and 41 minutes'
        }
    }

    saveFlightResults(payload)
    return json.dumps(payload)

def runFlightSearch(date, time, departureAirport=None):
    if departureAirport is None:
        departureAirport = "LAS"
    
    myLogger.debug("Calling runFlightSearch with date:{0}, time:{1}, departureAirport:{2}".format(date, time, departureAirport))
    flights = getFlights(departureAirport, date, time )

    def getAirportName(code):
        for airport in flights['airports']:
            if airport['fs'] == code:
                return airport['name'] + ", " + airport['stateCode'] if 'stateCode' in airport else airport['countryName']

    payload = {'flights':[]}

    for scheduledFlight in flights['scheduledFlights']:
        flight={}
        flight['flightnumber'] = scheduledFlight['carrierFsCode'] + ' ' + scheduledFlight['flightNumber']
        flight['time'] = scheduledFlight['departureTime']
        flight['timeUTC'] = scheduledFlight['departureTimeUTC']
        flight['destAirportCode'] = scheduledFlight['arrivalAirportFsCode']
        flight['destination'] = getAirportName(scheduledFlight['arrivalAirportFsCode'])
        payload['flights'].append(flight)
    #payload = {
    #    'flights': [
    #        {'flightnumber': 'CVC 0811', 'time': '6:15pm', 'destination': 'Newark, NY'},
    #        {'flightnumber': 'VA 0315', 'time': '6:30pm', 'destination': 'Boston, MA'} 
    #    ]
    #}
    return json.dumps(payload)



# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------

from pixiedust.display.display import *
from pixiedust.display import *
from .flightPredict import *
import pixiedust
import pixiedust.utils.dataFrameMisc as dataFrameMisc
from pixiedust.utils.shellAccess import ShellAccess
from pyspark.rdd import RDD
from pyspark.sql import DataFrame
from pyspark.mllib.regression import LabeledPoint
from six import iteritems, with_metaclass

myLogger = pixiedust.getLogger(__name__)
initialAirport = "BOS"

@PixiedustDisplay()
class PixieDustFlightPredictPluginMeta(DisplayHandlerMeta):
  def createCategories(self):
    return [{"id":"FlightPredict","title":"Flight Predictor", "icon-path":"flightPredict.jpeg"}]
  @addId
  def getMenuInfo(self,entity, dataHandler):
    if entity==self.__class__:
      return [{"id": "flightpredict"}]
    elif entity == "fp_configure_training":
      return [{"id": "fp_configure_training"}]
    elif entity == "fp_map_results":
      return [{"id": "fp_map_results"}]

    menus = []
    dataSetsValues = Configuration.getDataSets()
    dataSetsValues = dataSetsValues if len(dataSetsValues)==0 else list(zip(*dataSetsValues))[1]
    if entity in dataSetsValues:
      menus = menus + [
        {"categoryId": "FlightPredict", "title": "Visualize Features", "icon-path":"vizFeatures.png", "id":"fp_viz_features"},
        {"categoryId": "FlightPredict", "title": "Show Histogram", "icon-path":"vizFeatures.png", "id":"fp_histogram"}
      ]
      if len(Configuration.getModels())>0 and Configuration.getLabeledData(entity) is not None:
        menus.append( 
          {"categoryId": "FlightPredict", "title": "Measure Accuracy", "icon-path":"vizFeatures.png", "id":"fp_run_metrics"}
        )

    return menus

  def isLabeledRDD(self, entity):
    if isinstance(entity,RDD):
      sample = entity.take(1)
      if sample is not None and len(sample)>0:
        return isinstance(sample[0], LabeledPoint)
    return False

  def newDisplayHandler(self,options,entity):
    handlerId=options.get("handlerId")
    myLogger.debug("Creating a new Display Handler with id {0}".format(handlerId))
    if handlerId == "fp_viz_features":
      from . import vizFeatures
      return vizFeatures.VizualizeFeatures(options,entity)
    elif handlerId == "fp_configure_training":
      from . import configureTraining
      return configureTraining.ConfigureTraining(options,entity)
    elif handlerId == "fp_create_models":
      from . import createModels
      return createModels.CreateModels(options, entity)
    elif handlerId == "fp_histogram":
      from . import histogramDisplay
      return histogramDisplay.HistogramDisplay(options, entity)
    elif handlerId == "fp_run_metrics":
      from . import runMetrics
      return runMetrics.RunMetricsDisplay(options, entity)
    elif handlerId == "fp_map_results":
      from . import mapResults
      return mapResults.MapResultsDisplay(options, entity)
    else:
      options["initialAirport"] = initialAirport
      return PixieDustFlightPredict(options,entity)

def flightPredict(depAir="BOS"):
  global initialAirport
  initialAirport = depAir
  display(PixieDustFlightPredictPluginMeta)

def displayMapResults():
  display("fp_map_results")

def configure():
  display("fp_configure_training")

class Configuration(with_metaclass( 
        type("",(type,),{
            "configDict":{},
            "__getitem__":lambda cls, key: cls.configDict.get(key),
            "__setitem__":lambda cls, key,val: cls.configDict.update({key:val}),
            "__getattr__":lambda cls, key: cls.configDict.get(key),
            "__setattr__":lambda cls, key, val: cls.configDict.update({key:val})
        }), object
    )):
    
  @staticmethod
  def update(**kwargs):
    for key,val in iteritems(kwargs):
      Configuration[key]=val

  @staticmethod
  def getModels():
    return [(x,ShellAccess[x]) for x in ShellAccess if hasattr(ShellAccess[x], "predict") and callable(getattr(ShellAccess[x], "predict"))]

  @staticmethod
  def getDataSets():
    return [(x,ShellAccess[x]) for x in ShellAccess if (x=="trainingData" or x=="testData") and isinstance(ShellAccess[x], DataFrame)]

  @staticmethod
  def getLabeledData(entity):
    if ShellAccess[Configuration.DFTrainingVarName] == entity and ShellAccess[Configuration.LabeledRDDTrainingVarName] is not None:
      return (ShellAccess[Configuration.LabeledRDDTrainingVarName], Configuration.TrainingSQLTableName)
    elif ShellAccess[Configuration.DFTestVarName] == entity and ShellAccess[Configuration.LabeledRDDTestVarName] is not None:
      return (ShellAccess[Configuration.LabeledRDDTestVarName], Configuration.TestSQLTableName)
    return None

  @staticmethod
  def isReadyForRun():
    return len(Configuration.getModels())>0 and Configuration.weatherUrl is not None

def loadDataSet(dbName,sqlTable):
  if Configuration.cloudantHost is None or Configuration.cloudantUserName is None or Configuration.cloudantPassword is None:
    raise Exception("Missing credentials")
  cloudantdata = get_ipython().user_ns.get("sqlContext").read.format("com.cloudant.spark")\
    .option("cloudant.host",Configuration.cloudantHost)\
    .option("cloudant.username",Configuration.cloudantUserName)\
    .option("cloudant.password",Configuration.cloudantPassword)\
    .option("schemaSampleSize", "-1")\
    .load(dbName)

  cloudantdata.cache()
  print("Successfully cached dataframe")
  cloudantdata.registerTempTable(sqlTable)
  print("Successfully registered SQL table " + sqlTable);
  return cloudantdata


# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------
import requests
from datetime import datetime
from dateutil import parser
import pixiedust_flightpredict
import pixiedust

myLogger = pixiedust.getLogger(__name__)

def getWeather(airportLat, airportLong, dtString):
    weatherUrl = pixiedust_flightpredict.Configuration.weatherUrl
    myLogger.debug("Using weatherUrl {0}".format(weatherUrl))
    #weatherUrl="https://4b88408f-11e5-4ddc-91a6-fbd442e84879:p6hxeJsfIb@twcservice.mybluemix.net"
    dt=parser.parse(dtString)

    # weather insights
    #url=weatherUrl +'/api/weather/v2/forecast/hourly/24hour'
    #forecasts=requests.get(url, params=[('geocode',str(airportLat)+','+str(airportLong)),('units','m'),('language','en-US')]).json()['forecasts']

    # weather company data
    url=weatherUrl + '/api/weather/v1/geocode/{lat}/{long}/forecast/hourly/48hour.json'.format(lat=str(airportLat),long=str(airportLong))
    forecastsresp=requests.get(url, params=[('units','m'),('language','en-US')])
    myLogger.debug(forecastsresp.content)
    forecasts = forecastsresp.json()['forecasts']

    #find the forecasts that is closest to dt.tm_hour
    weatherForecast=None
    for f in forecasts:
        ftime=datetime.fromtimestamp(f['fcst_valid'])
        if ftime.hour==dt.hour:
            weatherForecast=f
    return weatherForecast


# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------

from pixiedust.display.display import *
from pyspark.sql import functions as F
import yaml
from pixiedust_flightpredict.running.flightHistory import loadFlightHistory
import pixiedust
from pixiedust.utils.shellAccess import ShellAccess

myLogger = pixiedust.getLogger(__name__)

class MapResultsDisplay(Display):

    def doRender(self, handlerId):
        self.addProfilingTime = False
        self._addScriptElement("https://d3js.org/d3.v3.js", checkJSVar="d3")
        #self._addScriptElement("https://mbostock.github.io/d3/talk/20111116/d3/d3.geo.js")
        #self._addScriptElement("https://cdnjs.cloudflare.com/ajax/libs/d3-geo-projection/0.2.16/d3.geo.projection.js")
        #self._addScriptElement("https://mbostock.github.io/d3/talk/20111116/d3/d3.geom.js")

        #Load the data from the flight history db
        df = loadFlightHistory()
        ShellAccess.flightHistoryDF = df

        res = df.flatMap(lambda row: [\
                (row.depAirportFSCode, row.depAirportName.encode("ascii","ignore"), 0.0 if row.depAirportLat is None else row.depAirportLat,0.0 if row.depAirportLong is None else row.depAirportLong), \
                (row.arrAirportFSCode, row.arrAirportName.encode("ascii","ignore"), 0.0 if row.arrAirportLat is None else row.arrAirportLat,0.0 if row.arrAirportLong is None else row.arrAirportLong)\
            ])\
            .distinct()\
            .map(lambda t: """ "{0}":{{"id":"{0}","name":"{1}","latitude":{2},"longitude":{3}}}""".format(t[0], t[1], t[2],t[3]))

        graphNodesJson="{"
        for r in res.collect():
            graphNodesJson+=("," if len(graphNodesJson)>1 else "") + str(r)
        graphNodesJson+="}"
        myLogger.debug("graphNodesJson: {0}".format(graphNodesJson))        

        graphLinksJson = df.select("arrAirportFSCode","depAirportFSCode")\
            .withColumnRenamed("depAirportFSCode", "src").withColumnRenamed("arrAirportFSCode", "dst")\
            .groupBy("src","dst").agg(F.count("src").alias("count"))\
            .toJSON().map(lambda j: yaml.safe_load(j)).collect()
        myLogger.debug("graphLinksJson: {0}".format(graphLinksJson))


        self._addHTMLTemplate("mapResults.html", graphNodesJson=graphNodesJson, graphLinksJson=graphLinksJson)


# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------

import pixiedust_flightpredict.training as training
from pixiedust.display.chart.renderers.baseChartDisplay import BaseChartDisplay
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.cm as cm
from pyspark.sql import Row
from functools import reduce
import pixiedust

myLogger = pixiedust.getLogger(__name__)

def makeList(l):
    return l if isinstance(l, list) else [l]

class VizualizeFeatures(BaseChartDisplay):

    def doRender(self, handlerId):
        f1="departureWeather.temp"
        f2="arrivalWeather.temp"
        f1=f1.split(".")
        f2=f2.split(".")
        handler=training.getTrainingHandler()
        darr=self.entity.rdd.map(lambda s: ( handler.computeClassification(s),(\
            reduce(lambda x,y: getattr(x,y) if isinstance(x, Row) else getattr(getattr(s,x),y), f1) if len(f1)>1 else getattr(s,f1[0]),\
            reduce(lambda x,y: getattr(x,y) if isinstance(x, Row) else getattr(getattr(s,x),y), f2) if len(f2)>1 else getattr(s,f2[0])\
            )))\
            .reduceByKey(lambda x,y: makeList(x) + makeList(y))\
            .collect()
        numClasses=handler.numClasses()
        citer=iter(cm.rainbow(np.linspace(0, 1, numClasses)))
        colors = [next(citer) for i in range(0, numClasses)]
        legends= [handler.getClassLabel(i) for i in range(0,numClasses)]
        sets=[]
        fig, ax = plt.subplots(figsize=(12,8))
        for t in darr:
            sets.append((ax.scatter([x[0] for x in t[1]],[x[1] for x in t[1]],color=colors[t[0]],alpha=0.5),legends[t[0]]))

        ax.set_ylabel("Departure Airport Temp")
        ax.set_xlabel("Arrival Airport Temp")
        ax.legend([x[0] for x in sets],
                [x[1] for x in sets],
                scatterpoints=1,
                loc='lower left',
                ncol=numClasses,
                fontsize=12)

    def doRenderChart(self):
        pass


# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------
from pixiedust.utils.shellAccess import ShellAccess
from pyspark.mllib.regression import LabeledPoint
from pyspark.mllib.linalg import Vectors
import pixiedust_flightpredict
import pixiedust

myLogger = pixiedust.getLogger(__name__)

__all__ = ['defaultTrainingHandler', 'loadLabeledDataRDD', 'getNumClasses', 'getTrainingHandler', 'attributes']

attributes=['dewPt','rh','vis','wc',
    #'wdir',
    'wspd','feels_like','uv_index']
attributesMsg = ['Dew Point', 'Relative Humidity', 'Prevailing Hourly visibility', 'Wind Chill', 
     #'Wind direction',
    'Wind Speed','Feels Like Temperature', 'Hourly Maximum UV Index']

#Display Confusion Matrix as an HTML table when computing metrics
displayConfusionTable=False

def buildLabeledPoint(s, classification, handler):
    features=[]
    for attr in attributes:
        features.append(getattr(s, attr + '_1'))
    for attr in attributes:
        features.append(getattr(s, attr + '_2'))
    customFeatures=handler.customTrainingFeatures(s)
    for v in customFeatures:
        features.append(v)
    return LabeledPoint(classification,Vectors.dense(features))

#default training handler class
class defaultTrainingHandler:
    defTrainingHandler = None
    def getClassLabel(self, value):
        if ( int(value)==0 ):
            return "Canceled"
        elif (int(value)==1 ):
            return "On Time"
        elif (int(value) == 2 ):
            return "Delayed less than 2 hours"
        elif (int(value) == 3 ):
            return "Delayed between 2 and 4 hours"
        elif (int(value) == 4 ):
            return "Delayed more than 4 hours"
        return value
        
    def numClasses(self):
        return 5
    
    def computeClassification(self, s):
        return s.classification
    
    def customTrainingFeaturesNames(self ):
        return []
    
    def customTrainingFeatures(self, s):
        return []
    
def getTrainingHandler():
    customHandlers = [ShellAccess[x] for x in ShellAccess if isinstance(ShellAccess[x], defaultTrainingHandler)]
    customTrainingHandler =  None if len(customHandlers)==0 else customHandlers[0]
    if customTrainingHandler is None:
        myLogger.debug("Using default customTrainingHandler")
        if defaultTrainingHandler.defTrainingHandler is None:
            defaultTrainingHandler.defTrainingHandler = defaultTrainingHandler()
        customTrainingHandler=defaultTrainingHandler.defTrainingHandler
    return customTrainingHandler

def getNumClasses():
    return getTrainingHandler().numClasses()
    
def loadLabeledDataRDD(sqlTable):    
    select = 'select '
    comma=''
    for attr in attributes:
        select += comma + 'departureWeather.' + attr + ' as ' + attr + '_1'
        comma=','
    select += ',deltaDeparture'
    select += ',classification'
    for attr in attributes:
        select += comma + 'arrivalWeather.' + attr + ' as ' + attr + '_2'
    
    for attr in getTrainingHandler().customTrainingFeaturesNames():
        select += comma + attr
    select += ' from ' + sqlTable
    
    df = ShellAccess.sqlContext.sql(select)

    handler=getTrainingHandler()
    datardd = df.map(lambda s: buildLabeledPoint(s, handler.computeClassification(s), handler))
    datardd.cache()
    return datardd




# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------

from .training import *




# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------

import requests
import os
import pixiedust
import pytz
from dateutil.parser import parse
import datetime
from pixiedust_flightpredict import Configuration

myLogger=pixiedust.getLogger(__name__)

flexBaseUrl = "https://api.flightstats.com/flex/"

def buildUrl(path, **kwargs):
    appId=os.environ.get("appId") or Configuration["appId"]
    appKey=os.environ.get("appKey") or Configuration["appKey"]

    if appId is None or appKey is None:
        raise ValueError("appId or appKey is not defined")
    return (flexBaseUrl+path).format(**kwargs) + "?appId={0}&appKey={1}".format(appId, appKey)

def toUTC(sDateTime, timeZoneRegionName):
    return pytz.timezone( timeZoneRegionName ).localize(parse(sDateTime)).astimezone (pytz.utc)

airportCache={}
def getAirportInfoFromFlightStats(code):
    if code in airportCache:
        return airportCache[code]

    airportPath = "airports/rest/v1/json/{code}/today"
    url = buildUrl(airportPath, code=code)
    myLogger.debug("Calling airport api with url: " + url)
    response = requests.get(url)
    if response.status_code != 200:
        msg = "Error while trying to get airport info for {0}. Error is {1}".format(code, str(response.reason))
        myLogger.error(msg)
        raise requests.HTTPError(msg, response=response)
    
    airportCache[code] = response.json()["airport"];
    return airportCache[code]

def parseDate(date, departureAirport = None):
    dt = pytz.utc.localize(datetime.datetime.utcfromtimestamp(date/1000))
    if departureAirport:
        info = getAirportInfoFromFlightStats( departureAirport )
        dt = dt.astimezone(pytz.timezone(info['timeZoneRegionName']))
    return (dt.year,dt.month,dt.day,dt.hour)

def getFlightSchedule(flight, date, departureAirport):
    myLogger.debug("getFlightSchedule with args: {0}, {1}".format( flight, date ) )
    #parse the flight and date
    index = flight.rfind(" ")
    carrier = flight[:index]
    flightnumber = flight[index+1:]

    (year,month,day,hour) = parseDate(date, departureAirport)
    (yearDT,monthDT,dayDT,hourDT) = parseDate(date)

    schedulesPath = "schedules/rest/v1/json/flight/{carrier}/{flightnumber}/departing/{year}/{month}/{day}"
    url = buildUrl(schedulesPath, carrier=carrier, flightnumber=flightnumber, year=year, month=month, day=day)
    myLogger.debug("Calling flight stats with url: " + url)
    response = requests.get( url )
    if response.status_code != 200:
        msg = "Error while trying to get schedule for flight {0}. Error is {1}".format(flight, str(response.reason))
        myLogger.error(msg)
        raise requests.HTTPError(msg, response=response)
        
    payload = response.json()

    def findAirport(airportCode):
        for airport in payload['appendix']['airports']:
            if airport['fs'] == airportCode:
                return airport
        myLogger.error("error find airport {0} from getFlightSchedule".format(airportCode))

    def findEquipment(code):
        for equipment in payload['appendix']['equipments']:
            if equipment['iata'] == code:
                return equipment
        myLogger.error("error find equipment {0} from getFlightSchedule".format(code))
        return {}

    def findAirline(code):
        for airline in payload['appendix']['airlines']:
            if airline['fs'] == code:
                return airline
        myLogger.error("error find airline {0} from getFlightSchedule".format(code))
        return {}

    #find the right flight as there may be more than one
    scheduledFlights = payload.pop("scheduledFlights")
    thisFlight = None
    if len(scheduledFlights)>1:
        for scheduledFlight in scheduledFlights:
            airport = findAirport(scheduledFlight['departureAirportFsCode'])
            if airport is not None:
                utcDT = toUTC( scheduledFlight["departureTime"], airport['timeZoneRegionName'])
                myLogger.info("Comparing time for airport {0} between {1} and {2}".format( airport['name'], utcDT.hour, hourDT))
                if utcDT.hour == hourDT:
                    thisFlight = scheduledFlight
                    thisFlight['departureTimeUTC'] = str(utcDT)
                    arrAirport = findAirport( scheduledFlight['arrivalAirportFsCode'])
                    thisFlight['arrivalTimeUTC'] = str( toUTC(scheduledFlight['arrivalTime'], arrAirport['timeZoneRegionName']))
                    break
    else:
        thisFlight = scheduledFlights[0]

    if thisFlight:
        airport = findAirport(thisFlight['departureAirportFsCode'])
        thisFlight['departureTimeUTC'] = str(toUTC( thisFlight["departureTime"], airport['timeZoneRegionName']))
        arrAirport = findAirport( thisFlight['arrivalAirportFsCode'])
        thisFlight['arrivalTimeUTC'] = str( toUTC(thisFlight['arrivalTime'], arrAirport['timeZoneRegionName']))
        myLogger.info("Found flight corresponding to flight {0}: {1}".format(flight, thisFlight))
        payload['scheduledFlight']=thisFlight

        #find equipment and airline info for this flight
        thisFlight['equipmentInfo']=findEquipment(thisFlight['flightEquipmentIataCode'])
        thisFlight['airlineInfo']=findAirline(thisFlight['carrierFsCode'])
    else:
        raise Exception("Unable to find flight corresponding to flight {0}".format(flight))

    payload.pop('request')
    return payload

flightsCache={}
def getFlights(airport, date, hour):
    parts=date.split("-")
    if len(parts)!=3:
        raise ValueError("Invalid date {0}".format(date))

    (year,month,day) = (parts[0], parts[1], parts[2])
    #check the cache first
    if len(hour)>2:
        hour = hour[:2]
    key = airport+date+str(hour)
    if key in flightsCache:
        return flightsCache[key]

    path = "schedules/rest/v1/json/from/{departureAirportCode}/departing/{year}/{month}/{day}/{hourOfDay}"

    url = buildUrl(path, departureAirportCode=airport, year=year,month=month,day=day,hourOfDay=hour)
    myLogger.debug("Calling getFlights with url: " + url)
    response = requests.get( url )
    if response.status_code != 200:
        msg = "Error while trying to get flights for airport {0} at hour {1}. Error is {2}".format(airport, hour, str(response.reason))
        myLogger.error(msg)
        raise requests.HTTPError(msg, response=response)

    payload = response.json()
    if "error" in payload:
        msg = "Error while trying to get flights for airport {0} at hour {1}. Error is {2}".format(airport, hour, payload['error']['errorMessage'])
        myLogger.error(msg)
        raise requests.HTTPError(msg, response = response)

    #convert departuretimes from local to UTC
    def findAirport(airportCode):
        for airport in payload['appendix']['airports']:
            if airport['fs'] == airportCode:
                return airport
    for flight in payload['scheduledFlights']:
        airport = findAirport( flight["departureAirportFsCode"] )
        if airport is not None:
            dt = parse(flight["departureTime"])
            flight["departureTimeUTC"] = str(pytz.timezone(airport['timeZoneRegionName']).localize(dt).astimezone (pytz.utc))
        else:
            myLogger.error("Unable to resolve airport code {0} because it is not in the appendix returned by flightstats".format(flight["departureAirportFsCode"]))

    ret = {
        "scheduledFlights": payload['scheduledFlights'],
        "airports": payload['appendix']['airports'] if 'airports' in payload['appendix'] else [],
        "airlines": payload['appendix']['airlines'] if 'airlines' in payload['appendix'] else []
    }
    flightsCache[key]=ret
    return ret






# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------
from pixiedust.display.display import Display
from pixiedust.display.chart.renderers.baseChartDisplay import BaseChartDisplay
import matplotlib
import matplotlib.pyplot as plt
import numpy as np
from pixiedust.utils.shellAccess import ShellAccess

class HistogramDisplay(BaseChartDisplay):
    def doRender(self, handlerId):
        rdd = ShellAccess.sqlContext.sql("select deltaDeparture from training").map(lambda s: s.deltaDeparture)\
                .filter(lambda s: s < 50 and s > 12)

        histo = rdd.histogram(50)
        bins = [i for i in histo[0]]

        fig, ax = plt.subplots(figsize=(12,8))

        ax.set_ylabel('Number of records')
        ax.set_xlabel('Bin')
        plt.title('Histogram')
        intervals = [abs(j-i) for i,j in zip(bins[:-1], bins[1:])]
        values=[sum(intervals[:i]) for i in range(0,len(intervals))]
        ax.bar(values, histo[1], intervals, color='b', label = "Bins")
        ax.set_xticks(bins[:-1],[int(i) for i in bins[:-1]])
        ax.legend()

    def doRenderChart(self):
        pass



from setuptools import setup,find_packages

setup(name='pixiedust_flightpredict',
      version='0.12',
      description='Flight delay predictor application with PixieDust',
      url='https://github.com/ibm-cds-labs/simple-data-pipe-connector-flightstats/tree/master/pixiedust_flightpredict',
      install_requires=['pixiedust'],
      author='David Taieb',
      author_email='david_taieb@us.ibm.com',
      license='Apache 2.0',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False)


# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2016
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------

from pixiedust.display.display import *
from pixiedust_flightpredict import Configuration
import pixiedust_flightpredict.training as training
from pyspark.mllib.evaluation import MulticlassMetrics

class RunMetricsDisplay(Display):
    def doRender(self, handlerId):
        html='<div class="pd_save"><table width=100%><tr><th>Model</th><th>Accuracy</th><th>Precision</th><th>Recall</th></tr>'
        confusionHtml = '<p>Confusion Tables for each Model</p>'
        for modelName,model in Configuration.getModels():
            label= model.__class__.__name__
            labeledDataRDD, sqlTableName = Configuration.getLabeledData(self.entity)
            predictionAndLabels = model.predict(labeledDataRDD.map(lambda lp: lp.features))
            metrics = MulticlassMetrics(\
                predictionAndLabels.zip(labeledDataRDD.map(lambda lp: lp.label)).map(lambda t: (float(t[0]),float(t[1])))\
            )
            html+='<tr><td>{0}</td><td>{1:.2f}%</td><td>{2:.2f}%</td><td>{3:.2f}%</td></tr>'\
                .format(label,metrics.weightedFMeasure(beta=1.0)*100, metrics.weightedPrecision*100,metrics.weightedRecall*100 )
            displayConfusionTable = True
            if ( displayConfusionTable ):
                #get labels from RDD
                handler=training.getTrainingHandler()
                classLabels = labeledDataRDD.map(lambda t: t.label).distinct().map(lambda l: handler.getClassLabel(l)).collect()
                confusionMatrix = metrics.call("confusionMatrix")
                confusionMatrixArray = confusionMatrix.toArray()
                #labels = metrics.call("labels")
                confusionHtml += "<p>" + label + "<p>"
                confusionHtml += "<table>"
                confusionHtml+="<tr><td></td>"
                for classLabel in classLabels:
                    confusionHtml+="<td>" + str(classLabel) + "</td>"
                confusionHtml+="</tr>"
                
                for i, row in enumerate(confusionMatrixArray):
                    confusionHtml += "<tr>"
                    confusionHtml += "<td>" + classLabels[i] + "</td>"
                    for j, cell in enumerate(row):
                        confusionHtml+="<td style='text-align:center'>" + ("<b>" if (i==j) else "") +  str(cell) + ("</b>" if (i==j) else "") + "</td>"
                    confusionHtml += "</tr>"
                confusionHtml += "</table>"
            
        html+='</table></div>'
        
        if ( displayConfusionTable ):
            html+=confusionHtml
        
        self._addHTML(html)